@@ -56,8 +56,12 @@ class HostState:
         self.tasks_child_state  = None
         self.rescue_child_state = None
         self.always_child_state = None
+        self.did_start_at_task  = False
 
     def __repr__(self):
+        return "HostState(%r)" % self._blocks
+
+    def __str__(self):
         def _run_state_to_string(n):
             states = ["ITERATING_SETUP", "ITERATING_TASKS", "ITERATING_RESCUE", "ITERATING_ALWAYS", "ITERATING_COMPLETE"]
             try:
@@ -76,7 +80,7 @@ class HostState:
                         ret.append(states[i])
                 return "|".join(ret)
 
-        return "HOST STATE: block=%d, task=%d, rescue=%d, always=%d, role=%s, run_state=%s, fail_state=%s, pending_setup=%s, tasks child state? %s, rescue child state? %s, always child state? %s" % (
+        return "HOST STATE: block=%d, task=%d, rescue=%d, always=%d, role=%s, run_state=%s, fail_state=%s, pending_setup=%s, tasks child state? %s, rescue child state? %s, always child state? %s, did start at task? %s" % (
             self.cur_block,
             self.cur_regular_task,
             self.cur_rescue_task,
@@ -88,21 +92,37 @@ class HostState:
             self.tasks_child_state,
             self.rescue_child_state,
             self.always_child_state,
+            self.did_start_at_task,
         )
 
+    def __eq__(self, other):
+        if not isinstance(other, HostState):
+            return False
+
+        for attr in (
+            '_blocks', 'cur_block', 'cur_regular_task', 'cur_rescue_task', 'cur_always_task',
+            'cur_role', 'run_state', 'fail_state', 'pending_setup', 'cur_dep_chain',
+            'tasks_child_state', 'rescue_child_state', 'always_child_state'
+            ):
+            if getattr(self, attr) != getattr(other, attr):
+                return False
+
+        return True
+
     def get_current_block(self):
         return self._blocks[self.cur_block]
 
     def copy(self):
         new_state = HostState(self._blocks)
-        new_state.cur_block        = self.cur_block
+        new_state.cur_block = self.cur_block
         new_state.cur_regular_task = self.cur_regular_task
-        new_state.cur_rescue_task  = self.cur_rescue_task
-        new_state.cur_always_task  = self.cur_always_task
-        new_state.cur_role         = self.cur_role
-        new_state.run_state        = self.run_state
-        new_state.fail_state       = self.fail_state
-        new_state.pending_setup    = self.pending_setup
+        new_state.cur_rescue_task = self.cur_rescue_task
+        new_state.cur_always_task = self.cur_always_task
+        new_state.cur_role = self.cur_role
+        new_state.run_state = self.run_state
+        new_state.fail_state = self.fail_state
+        new_state.pending_setup = self.pending_setup
+        new_state.did_start_at_task = self.did_start_at_task
         if self.cur_dep_chain is not None:
             new_state.cur_dep_chain = self.cur_dep_chain[:]
         if self.tasks_child_state is not None:
@@ -134,11 +154,20 @@ class PlayIterator:
         self._play = play
         self._blocks = []
 
+        # Default options to gather
+        gather_subset = C.DEFAULT_GATHER_SUBSET
+
+        # Retrieve subset to gather
+        if self._play.gather_subset is not None:
+            gather_subset = self._play.gather_subset
+
         setup_block = Block(play=self._play)
         setup_task = Task(block=setup_block)
         setup_task.action = 'setup'
         setup_task.tags   = ['always']
-        setup_task.args   = {}
+        setup_task.args   = {
+          'gather_subset': gather_subset,
+        }
         setup_task.set_loader(self._play._loader)
         setup_block.block = [setup_task]
 
@@ -173,7 +202,9 @@ class PlayIterator:
                         self.get_next_task_for_host(host)
 
                 # finally, reset the host's state to ITERATING_SETUP
-                self._host_states[host.name].run_state = self.ITERATING_SETUP
+                if start_at_matched:
+                    self._host_states[host.name].did_start_at_task = True
+                    self._host_states[host.name].run_state = self.ITERATING_SETUP
 
         if start_at_matched:
             # we have our match, so clear the start_at_task field on the
@@ -185,10 +216,13 @@ class PlayIterator:
         self._play.handlers.extend(play.compile_roles_handlers())
 
     def get_host_state(self, host):
-        try:
-            return self._host_states[host.name].copy()
-        except KeyError:
-            raise AnsibleError("invalid host (%s) specified for playbook iteration" % host)
+        # Since we're using the PlayIterator to carry forward failed hosts,
+        # in the event that a previous host was not in the current inventory
+        # we create a stub state for it now
+        if host.name not in self._host_states:
+            self._host_states[host.name] = HostState(blocks=[])
+
+        return self._host_states[host.name].copy()
 
     def get_next_task_for_host(self, host, peek=False):
 
@@ -241,6 +275,10 @@ class PlayIterator:
                 return (state, None)
 
             if state.run_state == self.ITERATING_SETUP:
+                # First, we check to see if we were pending setup. If not, this is
+                # the first trip through ITERATING_SETUP, so we set the pending_setup
+                # flag and try to determine if we do in fact want to gather facts for
+                # the specified host.
                 if not state.pending_setup:
                     state.pending_setup = True
 
@@ -255,105 +293,134 @@ class PlayIterator:
                     if (gathering == 'implicit' and implied) or \
                        (gathering == 'explicit' and boolean(self._play.gather_facts)) or \
                        (gathering == 'smart' and implied and not host._gathered_facts):
-                        # mark the host as having gathered facts
+                        # The setup block is always self._blocks[0], as we inject it
+                        # during the play compilation in __init__ above.
                         setup_block = self._blocks[0]
                         if setup_block.has_tasks() and len(setup_block.block) > 0:
                             task = setup_block.block[0]
-                        if not peek:
-                            host.set_gathered_facts(True)
+                            if not peek:
+                                # mark the host as having gathered facts, because we're
+                                # returning the setup task to be executed
+                                host.set_gathered_facts(True)
                 else:
+                    # This is the second trip through ITERATING_SETUP, so we clear
+                    # the flag and move onto the next block in the list while setting
+                    # the run state to ITERATING_TASKS
                     state.pending_setup = False
 
-                    state.cur_block += 1
-                    state.cur_regular_task = 0
-                    state.cur_rescue_task  = 0
-                    state.cur_always_task  = 0
                     state.run_state = self.ITERATING_TASKS
-                    state.child_state = None
+                    if not state.did_start_at_task:
+                        state.cur_block += 1
+                        state.cur_regular_task = 0
+                        state.cur_rescue_task  = 0
+                        state.cur_always_task  = 0
+                        state.child_state = None
 
             elif state.run_state == self.ITERATING_TASKS:
                 # clear the pending setup flag, since we're past that and it didn't fail
                 if state.pending_setup:
                     state.pending_setup = False
 
-                if state.fail_state & self.FAILED_TASKS == self.FAILED_TASKS:
-                    state.run_state = self.ITERATING_RESCUE
-                elif state.cur_regular_task >= len(block.block):
-                    state.run_state = self.ITERATING_ALWAYS
+                # First, we check for a child task state that is not failed, and if we
+                # have one recurse into it for the next task. If we're done with the child
+                # state, we clear it and drop back to geting the next task from the list.
+                if state.tasks_child_state:
+                    (state.tasks_child_state, task) = self._get_next_task_from_state(state.tasks_child_state, host=host, peek=peek)
+                    if self._check_failed_state(state.tasks_child_state):
+                        # failed child state, so clear it and move into the rescue portion
+                        state.tasks_child_state = None
+                        self._set_failed_state(state)
+                    else:
+                        # get the next task recursively
+                        if task is None or state.tasks_child_state.run_state == self.ITERATING_COMPLETE:
+                            # we're done with the child state, so clear it and continue
+                            # back to the top of the loop to get the next task
+                            state.tasks_child_state = None
+                            continue
                 else:
-                    task = block.block[state.cur_regular_task]
-                    # if the current task is actually a child block, we dive into it
-                    if isinstance(task, Block) or state.tasks_child_state is not None:
-                        if state.tasks_child_state is None:
+                    # First here, we check to see if we've failed anywhere down the chain
+                    # of states we have, and if so we move onto the rescue portion. Otherwise,
+                    # we check to see if we've moved past the end of the list of tasks. If so,
+                    # we move into the always portion of the block, otherwise we get the next
+                    # task from the list.
+                    if self._check_failed_state(state):
+                        state.run_state = self.ITERATING_RESCUE
+                    elif state.cur_regular_task >= len(block.block):
+                        state.run_state = self.ITERATING_ALWAYS
+                    else:
+                        task = block.block[state.cur_regular_task]
+                        # if the current task is actually a child block, create a child
+                        # state for us to recurse into on the next pass
+                        if isinstance(task, Block) or state.tasks_child_state is not None:
                             state.tasks_child_state = HostState(blocks=[task])
                             state.tasks_child_state.run_state = self.ITERATING_TASKS
                             state.tasks_child_state.cur_role = state.cur_role
-                        (state.tasks_child_state, task) = self._get_next_task_from_state(state.tasks_child_state, host=host, peek=peek)
-                        if task is None:
-                            # check to see if the child state was failed, if so we need to
-                            # fail here too so we don't continue iterating tasks
-                            if state.tasks_child_state.fail_state != self.FAILED_NONE:
-                                state.fail_state |= self.FAILED_TASKS
-                            state.tasks_child_state = None
-                            state.cur_regular_task += 1
-                            continue
-                    else:
+                            # since we've created the child state, clear the task
+                            # so we can pick up the child state on the next pass
+                            task = None
                         state.cur_regular_task += 1
 
             elif state.run_state == self.ITERATING_RESCUE:
-                if state.fail_state & self.FAILED_RESCUE == self.FAILED_RESCUE:
-                    state.run_state = self.ITERATING_ALWAYS
-                elif state.cur_rescue_task >= len(block.rescue):
-                    if len(block.rescue) > 0:
-                        state.fail_state = self.FAILED_NONE
-                    state.run_state = self.ITERATING_ALWAYS
+                # The process here is identical to ITERATING_TASKS, except instead
+                # we move into the always portion of the block.
+                if state.rescue_child_state:
+                    (state.rescue_child_state, task) = self._get_next_task_from_state(state.rescue_child_state, host=host, peek=peek)
+                    if self._check_failed_state(state.rescue_child_state):
+                        state.rescue_child_state = None
+                        self._set_failed_state(state)
+                    else:
+                        if task is None or state.rescue_child_state.run_state == self.ITERATING_COMPLETE:
+                            state.rescue_child_state = None
+                            continue
                 else:
-                    task = block.rescue[state.cur_rescue_task]
-                    if isinstance(task, Block) or state.rescue_child_state is not None:
-                        if state.rescue_child_state is None:
+                    if state.fail_state & self.FAILED_RESCUE == self.FAILED_RESCUE:
+                        state.run_state = self.ITERATING_ALWAYS
+                    elif state.cur_rescue_task >= len(block.rescue):
+                        if len(block.rescue) > 0:
+                            state.fail_state = self.FAILED_NONE
+                        state.run_state = self.ITERATING_ALWAYS
+                    else:
+                        task = block.rescue[state.cur_rescue_task]
+                        if isinstance(task, Block) or state.rescue_child_state is not None:
                             state.rescue_child_state = HostState(blocks=[task])
                             state.rescue_child_state.run_state = self.ITERATING_TASKS
                             state.rescue_child_state.cur_role = state.cur_role
-                        (state.rescue_child_state, task) = self._get_next_task_from_state(state.rescue_child_state, host=host, peek=peek)
-                        if task is None:
-                            # check to see if the child state was failed, if so we need to
-                            # fail here too so we don't continue iterating rescue
-                            if state.rescue_child_state.fail_state != self.FAILED_NONE:
-                                state.fail_state |= self.FAILED_RESCUE
-                            state.rescue_child_state = None
-                            state.cur_rescue_task += 1
-                            continue
-                    else:
+                            task = None
                         state.cur_rescue_task += 1
 
             elif state.run_state == self.ITERATING_ALWAYS:
-                if state.cur_always_task >= len(block.always):
-                    if state.fail_state != self.FAILED_NONE:
-                        state.run_state = self.ITERATING_COMPLETE
+                # And again, the process here is identical to ITERATING_TASKS, except
+                # instead we either move onto the next block in the list, or we set the
+                # run state to ITERATING_COMPLETE in the event of any errors, or when we
+                # have hit the end of the list of blocks.
+                if state.always_child_state:
+                    (state.always_child_state, task) = self._get_next_task_from_state(state.always_child_state, host=host, peek=peek)
+                    if self._check_failed_state(state.always_child_state):
+                        state.always_child_state = None
+                        self._set_failed_state(state)
                     else:
-                        state.cur_block += 1
-                        state.cur_regular_task = 0
-                        state.cur_rescue_task  = 0
-                        state.cur_always_task  = 0
-                        state.run_state = self.ITERATING_TASKS
-                        state.child_state = None
+                        if task is None or state.always_child_state.run_state == self.ITERATING_COMPLETE:
+                            state.always_child_state = None
                 else:
-                    task = block.always[state.cur_always_task]
-                    if isinstance(task, Block) or state.always_child_state is not None:
-                        if state.always_child_state is None:
+                    if state.cur_always_task >= len(block.always):
+                        if state.fail_state != self.FAILED_NONE:
+                            state.run_state = self.ITERATING_COMPLETE
+                        else:
+                            state.cur_block += 1
+                            state.cur_regular_task = 0
+                            state.cur_rescue_task  = 0
+                            state.cur_always_task  = 0
+                            state.run_state = self.ITERATING_TASKS
+                            state.tasks_child_state = None
+                            state.rescue_child_state = None
+                            state.always_child_state = None
+                    else:
+                        task = block.always[state.cur_always_task]
+                        if isinstance(task, Block) or state.always_child_state is not None:
                             state.always_child_state = HostState(blocks=[task])
                             state.always_child_state.run_state = self.ITERATING_TASKS
                             state.always_child_state.cur_role = state.cur_role
-                        (state.always_child_state, task) = self._get_next_task_from_state(state.always_child_state, host=host, peek=peek)
-                        if task is None:
-                            # check to see if the child state was failed, if so we need to
-                            # fail here too so we don't continue iterating always
-                            if state.always_child_state.fail_state != self.FAILED_NONE:
-                                state.fail_state |= self.FAILED_ALWAYS
-                            state.always_child_state = None
-                            state.cur_always_task += 1
-                            continue
-                    else:
+                            task = None
                         state.cur_always_task += 1
 
             elif state.run_state == self.ITERATING_COMPLETE:
@@ -366,7 +433,7 @@ class PlayIterator:
         return (state, task)
 
     def _set_failed_state(self, state):
-        if state.pending_setup:
+        if state.run_state == self.ITERATING_SETUP:
             state.fail_state |= self.FAILED_SETUP
             state.run_state = self.ITERATING_COMPLETE
         elif state.run_state == self.ITERATING_TASKS:
@@ -399,15 +466,24 @@ class PlayIterator:
 
     def mark_host_failed(self, host):
         s = self.get_host_state(host)
+        display.debug("marking host %s failed, current state: %s" % (host, s))
         s = self._set_failed_state(s)
+        display.debug("^ failed state is now: %s" % s)
         self._host_states[host.name] = s
 
     def get_failed_hosts(self):
-        return dict((host, True) for (host, state) in iteritems(self._host_states) if state.run_state == self.ITERATING_COMPLETE and state.fail_state != self.FAILED_NONE)
+        return dict((host, True) for (host, state) in iteritems(self._host_states) if self._check_failed_state(state))
 
     def _check_failed_state(self, state):
         if state is None:
             return False
+        elif state.fail_state != self.FAILED_NONE:
+            if state.run_state == self.ITERATING_RESCUE and state.fail_state&self.FAILED_RESCUE == 0:
+                return False
+            elif state.run_state == self.ITERATING_ALWAYS and state.fail_state&self.FAILED_ALWAYS == 0:
+                return False
+            else:
+                return True
         elif state.run_state == self.ITERATING_TASKS and self._check_failed_state(state.tasks_child_state):
             cur_block = self._blocks[state.cur_block]
             if len(cur_block.rescue) > 0 and state.fail_state & self.FAILED_RESCUE == 0:
@@ -418,13 +494,6 @@ class PlayIterator:
             return True
         elif state.run_state == self.ITERATING_ALWAYS and self._check_failed_state(state.always_child_state):
             return True
-        elif state.run_state == self.ITERATING_COMPLETE and state.fail_state != self.FAILED_NONE:
-            if state.run_state == self.ITERATING_RESCUE and state.fail_state&self.FAILED_RESCUE == 0:
-                return False
-            elif state.run_state == self.ITERATING_ALWAYS and state.fail_state&self.FAILED_ALWAYS == 0:
-                return False
-            else:
-                return True
         return False
 
     def is_failed(self, host):
@@ -438,7 +507,7 @@ class PlayIterator:
         the different processes, and not all data structures are preserved. This method
         allows us to find the original task passed into the executor engine.
         '''
-        def _search_block(block, task):
+        def _search_block(block):
             '''
             helper method to check a block's task lists (block/rescue/always)
             for a given task uuid. If a Block is encountered in the place of a
@@ -448,32 +517,32 @@ class PlayIterator:
             for b in (block.block, block.rescue, block.always):
                 for t in b:
                     if isinstance(t, Block):
-                        res = _search_block(t, task)
+                        res = _search_block(t)
                         if res:
                             return res
                     elif t._uuid == task._uuid:
                         return t
             return None
 
-        def _search_state(state, task):
+        def _search_state(state):
             for block in state._blocks:
-                res = _search_block(block, task)
+                res = _search_block(block)
                 if res:
                     return res
             for child_state in (state.tasks_child_state, state.rescue_child_state, state.always_child_state):
                 if child_state is not None:
-                    res = _search_state(child_state, task)
+                    res = _search_state(child_state)
                     if res:
                         return res
             return None
 
         s = self.get_host_state(host)
-        res = _search_state(s, task)
+        res = _search_state(s)
         if res:
             return res
 
         for block in self._play.handlers:
-            res = _search_block(block, task)
+            res = _search_block(block)
             if res:
                 return res
 
