@@ -561,6 +561,8 @@ def _human_to_bytes(number):
 
     if isinstance(number, int):
         return number
+    if number.isdigit():
+        return int(number)
     if number[-1] == suffixes[0] and number[-2].isdigit():
         return number[:-1]
 
@@ -1301,9 +1303,12 @@ class DockerManager(object):
                 expected_labels[name] = str(value)
 
             actual_labels = {}
-            for container_label in container['Config']['Labels'] or []:
-                name, value = container_label.split('=', 1)
-                actual_labels[name] = value
+            if isinstance(container['Config']['Labels'], dict):
+                actual_labels = container['Config']['Labels']
+            else:
+                for container_label in container['Config']['Labels'] or []:
+                    name, value = container_label.split('=', 1)
+                    actual_labels[name] = value
 
             if actual_labels != expected_labels:
                 self.reload_reasons.append('labels {0} => {1}'.format(actual_labels, expected_labels))
@@ -1852,7 +1857,7 @@ def main():
             volumes_from    = dict(default=None, type='list'),
             links           = dict(default=None, type='list'),
             devices         = dict(default=None, type='list'),
-            memory_limit    = dict(default=0, type='int'),
+            memory_limit    = dict(default=0),
             memory_swap     = dict(default=0, type='int'),
             cpu_shares      = dict(default=0, type='int'),
             docker_url      = dict(),
@@ -1871,7 +1876,7 @@ def main():
             domainname      = dict(default=None),
             env             = dict(type='dict'),
             env_file        = dict(default=None),
-            dns             = dict(),
+            dns             = dict(default=None, type='list'),
             detach          = dict(default=True, type='bool'),
             state           = dict(default='started', choices=['present', 'started', 'reloaded', 'restarted', 'stopped', 'killed', 'absent', 'running']),
             signal          = dict(default=None),
