@@ -5,7 +5,6 @@
 # (c) 2013, Dylan Martin <dmartin@seattlecentral.edu>
 # (c) 2015, Toshio Kuratomi <tkuratomi@ansible.com>
 # (c) 2016, Dag Wieers <dag@wieers.com>
-# (c) 2016, Virgil Dupras <hsoft@hardcoded.net>
 #
 # This file is part of Ansible
 #
@@ -114,9 +113,8 @@ import grp
 import datetime
 import time
 import binascii
-from zipfile import ZipFile
-import tarfile
-import subprocess
+import codecs
+from zipfile import ZipFile, BadZipfile
 
 # String from tar that shows the tar contents are different from the
 # filesystem
@@ -125,7 +123,7 @@ GROUP_DIFF_RE = re.compile(r': Gid differs$')
 MODE_DIFF_RE = re.compile(r': Mode differs$')
 #NEWER_DIFF_RE = re.compile(r' is newer or same age.$')
 MISSING_FILE_RE = re.compile(r': Warning: Cannot stat: No such file or directory$')
-ZIP_FILE_MODE_RE = re.compile(r'([r-][w-][stx-]){3}')
+ZIP_FILE_MODE_RE = re.compile(r'([r-][w-][SsTtx-]){3}')
 # When downloading an archive, how much of the archive to download before
 # saving to a tempfile (64k)
 BUFSIZE = 65536
@@ -165,17 +163,41 @@ class ZipArchive(object):
 #                    mode += 2**(9+j)
         return ( mode & ~umask )
 
+    def _legacy_file_list(self, force_refresh=False):
+        unzip_bin = self.module.get_bin_path('unzip')
+        if not unzip_bin:
+            raise UnarchiveError('Python Zipfile cannot read %s and unzip not found' % self.src)
+
+        rc, out, err = self.module.run_command([unzip_bin, '-v', self.src])
+        if rc:
+            raise UnarchiveError('Neither python zipfile nor unzip can read %s' % self.src)
+
+        for line in out.splitlines()[3:-2]:
+            fields = line.split(None, 7)
+            self._files_in_archive.append(fields[7])
+            self._infodict[fields[7]] = long(fields[6])
+
     def _crc32(self, path):
         if self._infodict:
             return self._infodict[path]
 
-        archive = ZipFile(self.src)
         try:
-            for item in archive.infolist():
-                self._infodict[item.filename] = long(item.CRC)
-        except:
-            archive.close()
-            raise UnarchiveError('Unable to list files in the archive')
+            archive = ZipFile(self.src)
+        except BadZipfile:
+            e = get_exception()
+            if e.args[0].lower().startswith('bad magic number'):
+                # Python2.4 can't handle zipfiles with > 64K files.  Try using
+                # /usr/bin/unzip instead
+                self._legacy_file_list()
+            else:
+                raise
+        else:
+            try:
+                for item in archive.infolist():
+                    self._infodict[item.filename] = long(item.CRC)
+            except:
+                archive.close()
+                raise UnarchiveError('Unable to list files in the archive')
 
         return self._infodict[path]
 
@@ -185,16 +207,26 @@ class ZipArchive(object):
             return self._files_in_archive
 
         self._files_in_archive = []
-        archive = ZipFile(self.src)
         try:
-            for member in archive.namelist():
-                if member not in self.excludes:
-                    self._files_in_archive.append(member)
-        except:
-            archive.close()
-            raise UnarchiveError('Unable to list files in the archive')
+            archive = ZipFile(self.src)
+        except BadZipfile:
+            e = get_exception()
+            if e.args[0].lower().startswith('bad magic number'):
+                # Python2.4 can't handle zipfiles with > 64K files.  Try using
+                # /usr/bin/unzip instead
+                self._legacy_file_list(force_refresh)
+            else:
+                raise
+        else:
+            try:
+                for member in archive.namelist():
+                    if member not in self.excludes:
+                        self._files_in_archive.append(member)
+            except:
+                archive.close()
+                raise UnarchiveError('Unable to list files in the archive')
 
-        archive.close()
+            archive.close()
         return self._files_in_archive
 
     def is_unarchived(self):
@@ -269,15 +301,18 @@ class ZipArchive(object):
         for line in old_out.splitlines():
             change = False
 
-            pcs = line.split()
-            if len(pcs) != 8: continue
+            pcs = line.split(None, 7)
+
+            # Check first and seventh field in order to skip header/footer
+            if len(pcs[0]) != 7 and len(pcs[0]) != 10: continue
+            if len(pcs[6]) != 15: continue
 
             ztype = pcs[0][0]
             permstr = pcs[0][1:10]
-            version = pcs[0][1]
-            ostype = pcs[0][2]
+            version = pcs[1]
+            ostype = pcs[2]
             size = int(pcs[3])
-            path = pcs[7]
+            path = unicode(pcs[7], 'utf-8')
 
             # Skip excluded files
             if path in self.excludes:
@@ -361,6 +396,7 @@ class ZipArchive(object):
                     elif stat.S_ISREG(st.st_mode) and timestamp < st.st_mtime:
                         # Add to excluded files, ignore other changes
                         out += 'File %s is newer, excluding file\n' % path
+                        self.excludes.append(path)
                         continue
                 else:
                     if timestamp != st.st_mtime:
@@ -387,16 +423,24 @@ class ZipArchive(object):
 
             # Do not handle permissions of symlinks
             if ftype != 'L':
+
+                # Use the new mode provided with the action, if there is one
+                if self.file_args['mode']:
+                    if isinstance(self.file_args['mode'], int):
+                        mode = self.file_args['mode']
+                    else:
+                        try:
+                            mode = int(self.file_args['mode'], 8)
+                        except Exception:
+                            e = get_exception()
+                            self.module.fail_json(path=path, msg="mode %(mode)s must be in octal form" % self.file_args, details=str(e))
                 # Only special files require no umask-handling
-                if ztype == '?':
+                elif ztype == '?':
                     mode = self._permstr_to_octal(permstr, 0)
                 else:
                     mode = self._permstr_to_octal(permstr, umask)
-                if self.file_args['mode'] and  self.file_args['mode'] != stat.S_IMODE(st.st_mode):
-                    change = True
-                    err += 'Path %s differs in permissions (%o vs %o)\n' % (path, self.file_args['mode'], stat.S_IMODE(st.st_mode))
-                    itemized[5] = 'p'
-                elif mode != stat.S_IMODE(st.st_mode):
+
+                if mode != stat.S_IMODE(st.st_mode):
                     change = True
                     itemized[5] = 'p'
                     err += 'Path %s differs in permissions (%o vs %o)\n' % (path, mode, stat.S_IMODE(st.st_mode))
@@ -458,11 +502,11 @@ class ZipArchive(object):
         cmd = '%s -o "%s"' % (self.cmd_path, self.src)
         if self.opts:
             cmd += ' ' + ' '.join(self.opts)
-        if self.includes:
-            cmd += ' "' + '" "'.join(self.includes) + '"'
-        # We don't need to handle excluded files, since we simply do not include them
-#        if self.excludes:
-#            cmd += ' -x ' + ' '.join(self.excludes)
+        # NOTE: Including (changed) files as arguments is problematic (limits on command line/arguments)
+#        if self.includes:
+#            cmd += ' "' + '" "'.join(self.includes) + '"'
+        if self.excludes:
+            cmd += ' -x ' + ' '.join(self.excludes)
         cmd += ' -d "%s"' % self.dest
         rc, out, err = self.module.run_command(cmd)
         return dict(cmd=cmd, rc=rc, out=out, err=err)
@@ -496,23 +540,25 @@ class TgzArchive(object):
         self.compress_mode = 'gz'
         self._files_in_archive = []
 
-    def _get_tar_fileobj(self):
-        """Returns a file object that can be read by ``tarfile.open()``."""
-        return open(self.src, 'rb')
-
     @property
     def files_in_archive(self, force_refresh=False):
         if self._files_in_archive and not force_refresh:
             return self._files_in_archive
 
-        # The use of Python's tarfile module here allows us to easily avoid tricky file encoding
-        # problems. Ref #11348
-        try:
-            tf = tarfile.open(fileobj=self._get_tar_fileobj(), mode='r:%s' % self.compress_mode)
-        except Exception:
+        cmd = '%s -t%s' % (self.cmd_path, self.zipflag)
+        if self.opts:
+            cmd += ' ' + ' '.join(self.opts)
+        if self.excludes:
+            cmd += ' --exclude="' + '" --exclude="'.join(self.excludes) + '"'
+        cmd += ' -f "%s"' % self.src
+        rc, out, err = self.module.run_command(cmd, environ_update=dict(LANG='C', LC_ALL='C', LC_MESSAGES='C'))
+        if rc != 0:
             raise UnarchiveError('Unable to list files in the archive')
 
-        for filename in tf.getnames():
+        for filename in out.splitlines():
+            # Compensate for locale-related problems in gtar output (octal unicode representation) #11348
+#            filename = filename.decode('string_escape')
+            filename = codecs.escape_decode(filename)[0]
             if filename and filename not in self.excludes:
                 self._files_in_archive.append(filename)
         return self._files_in_archive
@@ -532,7 +578,7 @@ class TgzArchive(object):
         if self.excludes:
             cmd += ' --exclude="' + '" --exclude="'.join(self.excludes) + '"'
         cmd += ' -f "%s"' % self.src
-        rc, out, err = self.module.run_command(cmd)
+        rc, out, err = self.module.run_command(cmd, environ_update=dict(LANG='C', LC_ALL='C', LC_MESSAGES='C'))
 
         # Check whether the differences are in something that we're
         # setting anyway
@@ -573,7 +619,7 @@ class TgzArchive(object):
         if self.excludes:
             cmd += ' --exclude="' + '" --exclude="'.join(self.excludes) + '"'
         cmd += ' -f "%s"' % (self.src)
-        rc, out, err = self.module.run_command(cmd, cwd=self.dest)
+        rc, out, err = self.module.run_command(cmd, cwd=self.dest, environ_update=dict(LANG='C', LC_ALL='C', LC_MESSAGES='C'))
         return dict(cmd=cmd, rc=rc, out=out, err=err)
 
     def can_handle_archive(self):
@@ -615,19 +661,6 @@ class TarXzArchive(TgzArchive):
         self.zipflag = 'J'
         self.compress_mode = ''
 
-    def _get_tar_fileobj(self):
-        # Python's tarfile module doesn't support xz compression so we have to manually uncompress
-        # it first.
-        xz_bin_path = self.module.get_bin_path('xz')
-        xz_stdout = tempfile.TemporaryFile()
-        # we don't use self.module.run_command() to avoid loading the whole archive in memory.
-        cmd = subprocess.Popen([xz_bin_path, '-dc', self.src], stdout=xz_stdout)
-        rc = cmd.wait()
-        if rc != 0:
-            raise UnarchiveError("Could not uncompress with xz")
-        xz_stdout.seek(0)
-        return xz_stdout
-
 
 # try handlers in order and return the one that works or bail if none work
 def pick_handler(src, dest, file_args, module):
@@ -650,10 +683,11 @@ def main():
             creates           = dict(required=False, type='path'),
             list_files        = dict(required=False, default=False, type='bool'),
             keep_newer        = dict(required=False, default=False, type='bool'),
-            exclude           = dict(requited=False, default=[], type='list'),
+            exclude           = dict(required=False, default=[], type='list'),
             extra_opts        = dict(required=False, default=[], type='list'),
         ),
         add_file_common_args = True,
+        # check-mode only works for zip files
 #        supports_check_mode = True,
     )
 
@@ -722,7 +756,7 @@ def main():
             if res_args['extract_results']['rc'] != 0:
                 module.fail_json(msg="failed to unpack %s to %s" % (src, dest), **res_args)
         except IOError:
-            module.fail_json(msg="failed to unpack %s to %s" % (src, dest))
+            module.fail_json(msg="failed to unpack %s to %s" % (src, dest), **res_args)
         else:
             res_args['changed'] = True
 
@@ -737,7 +771,7 @@ def main():
             try:
                 res_args['changed'] = module.set_fs_attributes_if_different(file_args, res_args['changed'])
             except (IOError, OSError), e:
-                module.fail_json(msg="Unexpected error when accessing exploded file: %s" % str(e))
+                module.fail_json(msg="Unexpected error when accessing exploded file: %s" % str(e), **res_args)
 
     if module.params['list_files']:
         res_args['files'] = handler.files_in_archive
