@@ -183,11 +183,11 @@ def find_subnets(vpc_conn, vpc_id, identified_subnets):
             filters={'vpc_id': vpc_id, 'tag:Name': subnet_names})
 
         for name in subnet_names:
-            matching = [s.tags.get('Name') == name for s in subnets_by_name]
-            if len(matching) == 0:
+            matching_count = len([1 for s in subnets_by_name if s.tags.get('Name') == name])
+            if matching_count == 0:
                 raise AnsibleSubnetSearchException(
                     'Subnet named "{0}" does not exist'.format(name))
-            elif len(matching) > 1:
+            elif matching_count > 1:
                 raise AnsibleSubnetSearchException(
                     'Multiple subnets named "{0}"'.format(name))
 
@@ -319,7 +319,8 @@ def ensure_routes(vpc_conn, route_table, route_specs, propagating_vgw_ids,
     # VGWs in place.
     routes_to_delete = [r for r in routes_to_match
                         if r.gateway_id != 'local'
-                        and r.gateway_id not in propagating_vgw_ids]
+                        and (propagating_vgw_ids is not None
+                             and r.gateway_id not in propagating_vgw_ids)]
 
     changed = routes_to_delete or route_specs_to_create
     if changed:
