@@ -123,7 +123,8 @@ class ActionModule(ActionBase):
             # loader, so that it knows about the other paths to find template files
             searchpath = [self._loader._basedir, os.path.dirname(source)]
             if self._task._role is not None:
-                searchpath.insert(1, C.DEFAULT_ROLES_PATH)
+                if C.DEFAULT_ROLES_PATH:
+                    searchpath[:0] = C.DEFAULT_ROLES_PATH
                 searchpath.insert(1, self._task._role._role_path)
 
             self._templar.environment.loader.searchpath = searchpath
@@ -137,10 +138,10 @@ class ActionModule(ActionBase):
             result['msg'] = type(e).__name__ + ": " + str(e)
             return result
 
-        cleanup_remote_tmp = False
+        remote_user = task_vars.get('ansible_ssh_user') or self._play_context.remote_user
         if not tmp:
-            tmp = self._make_tmp_path()
-            cleanup_remote_tmp = True
+            tmp = self._make_tmp_path(remote_user)
+            self._cleanup_remote_tmp = True
 
         local_checksum = checksum_s(resultant)
         remote_checksum = self.get_checksum(dest, task_vars, not directory_prepended, source=source, tmp=tmp)
@@ -163,8 +164,7 @@ class ActionModule(ActionBase):
                 xfered = self._transfer_data(self._connection._shell.join_path(tmp, 'source'), resultant)
 
                 # fix file permissions when the copy is done as a different user
-                if self._play_context.become and self._play_context.become_user != 'root':
-                    self._remote_chmod('a+r', xfered)
+                self._fixup_perms2((tmp, xfered), remote_user)
 
                 # run the copy module
                 new_module_args.update(
@@ -196,7 +196,6 @@ class ActionModule(ActionBase):
             )
             result.update(self._execute_module(module_name='file', module_args=new_module_args, task_vars=task_vars, tmp=tmp, delete_remote_tmp=False))
 
-        if tmp and cleanup_remote_tmp:
-            self._remove_tmp_path(tmp)
+        self._remove_tmp_path(tmp)
 
         return result
